const axios = require('axios');
const serverConfig = require('../../../config/ServerConfig.js');
const updaterModel = require('../model/updaterModel');
const adqGetDashboardFiliais = require('../model/adqGetDashboardFiliais');
const paginacao = require('../functions/Paginacao');
const OracleService = require('../service/OracleService');

class ApiController {
  async getOrgProducts() {
    let result = {
      isError: false,
      response: null
    };
    const connection = await OracleService.connect();
    try {
      const filter = {
        requestType : 'FilterData',
        filter : [ {
          'name' : 'NRORG', 'operator' : '=', 'value' : serverConfig.nrorg
        } ]
      };
      const axiosRep = await axios.post(`${serverConfig.licenseUrl}/getOrgProducts`, filter, { timeout: 30000 });
      if(axiosRep.data.dataset && axiosRep.data.dataset.products) {
        const products = axiosRep.data.dataset.products;
        const system = await updaterModel.getFullUpdaterSistemas(connection);
        let actualSystem = {}; let actualProducts = {}; let cdfilial;
        system.rows.forEach((upSystem) => {
          //Pos 3 = CDFILIAL; Pos 1 = DEVOPSID
          actualSystem[upSystem[3]] = actualSystem[upSystem[3]] || [];
          actualSystem[upSystem[3]].push(upSystem[1].toString());
        });
        let { rows } = await adqGetDashboardFiliais.get(paginacao());
        rows = [[9999]];
        //Insert system
        await Promise.all(rows.map(async (filial) => {
          //Pos 0 = CDFILIAL
          cdfilial = filial[0];
          await Promise.all(products.map(async (product) => {
            actualProducts[cdfilial] = actualProducts[cdfilial] || [];
            actualProducts[cdfilial].push(product.ID);
            if(!actualSystem[cdfilial] || !actualSystem[cdfilial].includes(product.ID)) {
              const params = {
                CDFILIAL:       cdfilial,
                DEVOPSID:       product.ID,
                SISTEMASSTATUS: 0,
                APLICACAO:      product.NMPRODUTO,
                DIRETORIO:      product.PATH_TO_DEPLOY
              };
              await updaterModel.setUpdaterSistemas(params, connection);
            }
          }));
        }));
        //Cancel system
        await Promise.all(rows.map(async (filial) => {
          //Pos 0 = CDFILIAL
          cdfilial = filial[0];
          if(actualSystem[cdfilial]) {
            await Promise.all(actualSystem[cdfilial].map(async (actSystemId) => {
              if(!actualProducts[cdfilial] || !actualProducts[cdfilial].includes(actSystemId)) {
                const params = {
                  CDFILIAL: cdfilial,
                  DEVOPSID: actSystemId
                };
                await updaterModel.setUpdaterSistemasCanceled(params, connection);
              }
            }));
          }
        }));
        await connection.commit();
        result.response = products;
      } else {
        connection.rollback();
        result.isError = true;
        if(axiosRep.data.error) {
          result.response = axiosRep.data.error;
        } else if(axiosRep.data.includes('Stack trace')) {
          result.response = axiosRep.data.split('Stack trace')[0];
        } else {
          result.response = axiosRep.data;
        }
      }
      return result;
    } catch (error) {
      connection.rollback();
      result.isError = true;
      console.log(' ----------- ERRO NO GET ORG PRODUCTS ---------- ');
      if(error.toJSON) {
        error = error.toJSON();
        console.log(error.message);
        result.response = error.message;
      } else {
        console.log(error);
        result.response = error;
      }
      return result;
    }
  }
}

module.exports = new ApiController();
