const OracleService = require('../service/OracleService');

class updaterModel {
  constructor() {
    /**
     * UPDATER
     */
    // SELECT * FROM FILIAL F INNER JOIN
    // UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
    // UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS)
    this.GET_UPDATER = `
      SELECT
      F.CDFILIAL,
      F.NMFILIAL,
      UPS.*,
      EN.SGESTADO
      FROM
        UPDATERSISTEMAS UPS INNER JOIN
        FILIAL F ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
        ENDEFILI EN ON (F.CDFILIAL = EN.CDFILIAL)
      WHERE EN.IDTPENDEFILI = 'P'
    `;

    /**
     * UPDATERSISTEMAS
     */
    this.GET_UPDATERSISTEMAS = `
     SELECT * FROM UPDATERSISTEMAS
      WHERE NOME = :NOME
   `;

    this.GET_FULL_UPDATERSISTEMAS = `
     SELECT IDUPDATERSISTEMAS, DEVOPSID, SISTEMASSTATUS, CDFILIAL FROM UPDATERSISTEMAS
   `;

    /**
     * UPDATERVERSAO
     */
    this.GET_UPDATERVERSAO = `
     SELECT * FROM UPDATERVERSAO
      WHERE IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND UPVERSTATUS = 0
   `;

    this.SET_UPDATERVERSAO = `
      INSERT INTO UPDATERVERSAO
        (IDUPDATERSISTEMAS, CDOPERADOR, UPVERDATA, UPVERSTATUS)
        VALUES (:IDUPDATERSISTEMAS, :CDOPERADOR, :UPVERDATA, :UPVERSTATUS)
   `;

    /**
     * Histórico
     */
    this.GET_UPDATER_HISTORICO = `
      SELECT OP.NMOPERADOR, OP.CDOPERADOR, UPR.*, UPS.*, UPV.*
        FROM FILIAL F INNER JOIN
          UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)
    `;
    /**
     * Histórico
     */
    this.GET_UPDATER_V_ATUAL = `
      SELECT
          F.CDFILIAL,
          F.NMFILIAL,
          OP.NMOPERADOR,
          OP.CDOPERADOR,
          UPS.*,
          UPV.*
        FROM FILIAL F INNER JOIN
          UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

        WHERE
          UPS.IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND
          UPV.UPVERSTATUS = 0 AND
          ROWNUM <= :MAX_ROW
        ORDER BY UPV.IDUPDATERVERSAO DESC
   `;

    this.GET_UPDATER_V_ALL = `
      SELECT
          F.CDFILIAL,
          F.NMFILIAL,
          OP.NMOPERADOR, OP.CDOPERADOR, UPS.*, UPV.*
        FROM FILIAL F INNER JOIN
          UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

        WHERE
          UPS.IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND
          UPS.CDFILIAL = :CDFILIAL AND
          ROWNUM <= :MAX_ROW
        ORDER BY UPV.IDUPDATERVERSAO DESC
   `;

   /**
    * Retorna os sistemas a serem atualizados para cilial cdFilial
    */
   this.GET_UPDATER_ESPERA_ALL = `
   SELECT UPS.APLICACAO, UPV.IDUPDATERVERSAO, UPS.IDUPDATERSISTEMAS, UPS.DIRETORIO
      FROM
        FILIAL F INNER JOIN
        UPDATERSISTEMAS UPS ON (F.CDFILIAL = UPS.CDFILIAL) INNER JOIN
        UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
        OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

      WHERE
        F.CDFILIAL = :cdfilial AND
        UPV.UPVERSTATUS = 0 AND
        UPS.DIRETORIO IS NOT NULL
      GROUP BY UPS.APLICACAO, UPV.IDUPDATERVERSAO, UPS.IDUPDATERSISTEMAS, UPS.DIRETORIO
   `;

   this.UPDATE_UPDATER = `
      UPDATE UPDATERVERSAO
        SET UPVERSTATUS = :UPVERSTATUS
      WHERE IDUPDATERVERSAO = :IDUPDATERVERSAO
   `;

   this.UPDATE_UPDATERSISTEMA_ATUAL = `
      UPDATE UPDATERSISTEMAS
        SET UPSISVERATUAL = :UPSISVERATUAL
      WHERE IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS
   `;

   this.SET_UPDATERSISTEMAS = `
    INSERT INTO UPDATERSISTEMAS
                (CDFILIAL,   DEVOPSID,  SISTEMASSTATUS,  APLICACAO,  DIRETORIO)
         VALUES (:CDFILIAL, :DEVOPSID, :SISTEMASSTATUS, :APLICACAO, :DIRETORIO)
  `;

   this.SET_UPDATERSISTEMAS_CANCELED = `
      UPDATE UPDATERSISTEMAS
         SET SISTEMASSTATUS = 2
       WHERE DEVOPSID = :DEVOPSID
         AND CDFILIAL = :CDFILIAL
  `;
  }

  getUpdaterVersao() {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER);
    });
  }

  getUpdaterVersaoAtual(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_V_ATUAL, params);
    });
  }

  getUpdaterVersaoAll(params) {
    return OracleService.connect().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_V_ALL, params);
    });
  }

  /**
   * @author Daniel Souza <daniel.souza@teknisa.com>
   * @param {*} params {cdfilial: 9999}
   * @returns Lista de Sistemas a serem atualizados na filial cdFilial
   */
  getVersaoDisponivel(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATER_ESPERA_ALL, params);
    });
  }

  updateVersaoDisponivel(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.UPDATE_UPDATER, params);
    });
  }

  updateVersaoAtual(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.UPDATE_UPDATERSISTEMA_ATUAL, params);
    });
  }

  setUpdaterVersao(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.SET_UPDATERVERSAO, params);
    });
  }

  setUpdaterSistemas(params, connection) {
    return OracleService.execute(this.SET_UPDATERSISTEMAS, params, [], false, connection);
  }

  setUpdaterSistemasCanceled(params, connection) {
    return OracleService.execute(this.SET_UPDATERSISTEMAS_CANCELED, params, [], false, connection);
  }

  getFullUpdaterSistemas(connection) {
    return OracleService.execute(this.GET_FULL_UPDATERSISTEMAS, [], [], false, connection);
  }

  /**
   * @author Daniel Souza <daniel.souza@teknisa.com>
   * @param {*} params {cdfilial: 9999}
   * @returns Retorna se o sistema já tem um pedido de update
   */
   getRegistroUpdate(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.fetchAssoc(this.GET_UPDATERVERSAO, params);
    });
  }
}

module.exports = new updaterModel();
